Package: somhca
Type: Package
Title: Self-Organising Maps Coupled with Hierarchical Cluster Analysis
Version: 0.4.0
Authors@R: person(given = "Gianluca",
                  family = "Pastorelli",
                  role = c("aut", "cre"),
                  email = "gianluca.pastorelli@gmail.com",
			comment = c(ORCID = "0000-0001-6926-1952"))
Description: Implements self-organising maps combined with hierarchical cluster analysis (SOM-HCA) for clustering and visualization of high-dimensional data.
    The package includes functions to estimate the optimal map size based on various quality measures
    and to generate a model using the selected dimensions.
    It also performs hierarchical clustering on the map nodes or other data to group similar units.
    Documentation about the SOM-HCA method is provided in Pastorelli et al. (2024)
    <doi:10.1002/xrs.3388>.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: kohonen, aweSOM, dplyr, RColorBrewer, grDevices, stats, utils,
        maptree, fpc
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-19 13:07:47 UTC; B041254
Author: Gianluca Pastorelli [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6926-1952>)
Maintainer: Gianluca Pastorelli <gianluca.pastorelli@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-19 13:20:02 UTC
Built: R 4.6.0; ; 2026-05-19 13:33:54 UTC; unix

Package: somhca
Type: Package
Title: Self-Organising Maps Coupled with Hierarchical Cluster Analysis
Version: 0.4.0
Authors@R: person(given = "Gianluca",
                  family = "Pastorelli",
                  role = c("aut", "cre"),
                  email = "gianluca.pastorelli@gmail.com",
			comment = c(ORCID = "0000-0001-6926-1952"))
Description: Implements self-organising maps combined with hierarchical cluster analysis (SOM-HCA) for clustering and visualization of high-dimensional data.
    The package includes functions to estimate the optimal map size based on various quality measures
    and to generate a model using the selected dimensions.
    It also performs hierarchical clustering on the map nodes or other data to group similar units.
    Documentation about the SOM-HCA method is provided in Pastorelli et al. (2024)
    <doi:10.1002/xrs.3388>.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: kohonen, aweSOM, dplyr, RColorBrewer, grDevices, stats, utils,
        maptree, fpc
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-19 13:07:47 UTC; B041254
Author: Gianluca Pastorelli [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6926-1952>)
Maintainer: Gianluca Pastorelli <gianluca.pastorelli@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-19 13:20:02 UTC
Built: R 4.5.2; ; 2026-05-19 15:45:49 UTC; unix

# Generated by roxygen2: do not edit by hand

export(clusterSOM)
export(clusterX)
export(finalSOM)
export(generatePlot)
export(getClusterData)
export(loadMatrix)
export(optimalSOM)
import(dplyr)
import(kohonen)
importFrom(RColorBrewer,brewer.pal)
importFrom(aweSOM,somQuality)
importFrom(fpc,cluster.stats)
importFrom(grDevices,colorRampPalette)
importFrom(kohonen,som)
importFrom(kohonen,somgrid)
importFrom(maptree,kgs)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,rect.hclust)
importFrom(stats,sd)
importFrom(utils,read.csv)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)

#' Perform Hierarchical Clustering on Data Objects
#'
#' Groups similar observations using hierarchical clustering
#' This function is designed for clustering data types other than trained SOM models.
#' By default, the optimal number of clusters is determined automatically using the KGS penalty function,
#' but the user can also specify a fixed number of clusters.
#'
#' @importFrom maptree kgs
#' @importFrom stats dist hclust cutree
#' @importFrom stats rect.hclust
#' @importFrom utils read.csv
#' @importFrom fpc cluster.stats
#' @param x Data frame or matrix containing numeric data to be clustered (non-numeric columns are automatically ignored). Argument is required.
#' @param n_clusters Integer.
#'   If provided, specifies the number of clusters to cut the dendrogram into.
#'   If NULL (default), the optimal number of clusters is determined automatically
#'   using the KGS penalty function.
#' @param validity_indices Logical. Indicates whether to compute and print popular clustering validity indices (Silhouette, Dunn, Calinski-Harabasz, Pearson Gamma). Default is `TRUE`.
#' @param plot_result Logical. Indicates whether to plot the clustering result. Default is `TRUE`.
#' @param input Character or in-memory dataset. A string specifying the path to a CSV file, or an in-memory
#'   object (data frame or matrix). Default is `NULL`.
#'   If provided, cluster assignments are appended to the observations in the original dataset,
#'   and the updated data is stored in a package environment as 'DataAndClusters'.
#' @return
#'   Invisibly returns `NULL`. If `plot_result = TRUE`, a plot of the clustering result is produced.
#'   If `input` is provided, the clustered dataset is stored in the package
#'   environment and can be retrieved with `getClusterData()`.
#' @examples
#' # Create a toy matrix with 9 columns and 100 rows
#' data <- matrix(rnorm(900), ncol = 9, nrow = 100)
#'
#' # Example 1: Perform clustering directly on the matrix
#' clusterX(data, plot_result = TRUE)
#'
#' # Example 2: Assign clusters to an in-memory data frame
#' df <- data.frame(
#'   ID = paste0("Sample", 1:100), # Character column for row headings
#'   matrix(rnorm(900), ncol = 9, nrow = 100) # Numeric data
#' )
#' clusterX(data, plot_result = FALSE, input = df)
#' getClusterData()
#' @export

clusterX <- function(x,
                     n_clusters = NULL,
                     validity_indices = TRUE,
                     plot_result = TRUE,
                     input = NULL) {

  # -----------------------------
  # Validate data
  # -----------------------------
  if (is.null(x)) {
    stop("`x` must be provided as a matrix or data frame.")
  }

  if (!(is.matrix(x) || is.data.frame(x))) {
    stop("`x` must be a data frame or matrix.")
  }

  # -----------------------------
  # Keep only numeric columns
  # -----------------------------
  if (is.data.frame(x)) {

    numeric_cols <- vapply(x, is.numeric, logical(1))

    if (!any(numeric_cols)) {
      stop("`x` must contain at least one numeric column.")
    }

    removed <- names(x)[!numeric_cols]

    if (length(removed) > 0) {
      message(
        "Ignoring non-numeric columns: ",
        paste(removed, collapse = ", ")
      )
    }

    x <- x[, numeric_cols, drop = FALSE]
  }

  x <- as.matrix(x)

  # -----------------------------
  # Hierarchical clustering
  # -----------------------------
  distance <- dist(x, method = "euclidean")
  clustering <- hclust(distance, method = "ward.D2")

  # -----------------------------
  # Determine number of clusters
  # -----------------------------
  if (is.null(n_clusters)) {

    optimal_k <- kgs(clustering, distance, maxclust = 20)
    clusters <- as.integer(names(optimal_k[which.min(optimal_k)]))

    message(clusters, " clusters were determined using KGS.\n")

  } else {

    if (!is.numeric(n_clusters) || length(n_clusters) != 1 || n_clusters < 1) {
      stop("`n_clusters` must be a single positive integer.")
    }

    clusters <- as.integer(n_clusters)
    message(clusters, " clusters were specified by the user.\n")
  }

  if (clusters > nrow(x)) {
    stop("`n_clusters` cannot exceed number of observations.")
  }

  som_cluster <- cutree(clustering, clusters)

  # -----------------------------
  # Validity indices
  # -----------------------------
  if (isTRUE(validity_indices)) {

    if (requireNamespace("fpc", quietly = TRUE)) {

      stats <- fpc::cluster.stats(distance, som_cluster)

      message(
        "Cluster validity indices:\n",
        sprintf("  %-25s %6.3f (higher is better; >0.25 reasonable, >0.5 strong)\n",
                "Average Silhouette Width:", stats$avg.silwidth),
        sprintf("  %-25s %6.3f (higher is better)\n",
                "Dunn Index:", stats$dunn),
        sprintf("  %-25s %6.1f (higher is better)\n",
                "Calinski-Harabasz Index:", stats$ch),
        sprintf("  %-25s %6.3f (closer to 1 is better)",
                "Pearson Gamma Coefficient:", stats$pearsongamma)
      )

    } else {
      message("Package 'fpc' not installed; skipping validity indices.")
    }
  }

  # -----------------------------
  # Plot
  # -----------------------------
  if (plot_result) {

    max_colors <- max(20, clusters)
    palette <- colorRampPalette(RColorBrewer::brewer.pal(8, "Set1"))(max_colors)

    plot(clustering,
         main = "Hierarchical Clustering Dendrogram",
         xlab = "Observations",
         ylab = "Dissimilarity",
         sub = ""
    )

    rect.hclust(clustering,
                k = clusters,
                border = palette[1:clusters])
  }

  # -----------------------------
  # Attach clusters to external data (optional)
  # -----------------------------
  if (!is.null(input)) {

    if (!is.data.frame(input)) {
      input <- as.data.frame(input)
    }

    if (nrow(input) != nrow(x)) {
      stop("`input` must have same number of rows as `x`.")
    }

    input$Cluster <- factor(som_cluster)
    input <- input[, c("Cluster", setdiff(names(input), "Cluster"))]

    somhca_env$DataAndClusters <- input

    message("The clustered dataset is stored in the package environment as 'DataAndClusters'. Use `getClusterData()` to retrieve it.")
  }

  invisible(som_cluster)
}

#' Re-Train SOM Model
#'
#' Re-trains the SOM using a specified optimal grid size and number of iterations.
#'
#' @importFrom kohonen som somgrid
#' @param data Matrix containing numeric data. A preprocessed data matrix containing the input data for SOM training. Argument is required.
#' @param dimension Integer. Dimension of the square SOM grid (e.g., 5 results in a 5x5 grid). Argument is required.
#' @param iterations Integer. Number of iterations for training the SOM model. Use a large value, e.g., 500 or higher, for improved training (an error message could suggest that reducing the number of iterations might be necessary).
#'   For larger grids, more iterations may be required to ensure convergence. Reducing iterations may speed training but risk under-trained neurons.  Argument is required.
#' @param chunk Integer. Number of iterations per training block.
#'   The SOM will be trained in chunks of this many iterations, with a progress
#'   message printed after each block. This helps notify the user that the function
#'   is running and not frozen. Larger values reduce the frequency
#'   of messages; smaller values provide more frequent updates but may slightly
#'   slow execution. Default is 100.
#' @return A trained SOM model object.
#' @examples
#' # Create a toy matrix with 9 columns and 100 rows
#' data <- matrix(rnorm(900), ncol = 9, nrow = 100)  # 900 random numbers, 100 rows, 9 columns
#'
#' # Run the finalSOM function with the mock data
#' myFinalSOM <- finalSOM(data, dimension = 6, iterations = 700)
#' @export

finalSOM <- function(data, dimension, iterations, chunk = 100) {

  # Basic safety checks
  stopifnot(is.numeric(dimension), dimension >= 2)
  stopifnot(is.numeric(iterations), iterations >= 1)
  stopifnot(is.numeric(chunk), chunk >= 1)

  message(sprintf("Training SOM: %dx%d grid, %d iterations", dimension, dimension, iterations))

  My_Grid <- somgrid(xdim = dimension, ydim = dimension, topo = "hexagonal", toroidal = TRUE)

  # Initialize SOM with 0 iterations
  My_Model <- som(X = data, grid = My_Grid, rlen = 0, alpha = c(0.05, 0.01), keep.data = TRUE)

  for (start in seq(1, iterations, by = chunk)) {
    end <- min(start + chunk - 1, iterations)

    # Train SOM for the next chunk
    My_Model <- som(X = data, grid = My_Grid, rlen = end, alpha = c(0.05, 0.01), keep.data = TRUE,
                    init = My_Model$codes)  # Continue from previous state

    message(sprintf("Completed %d / %d iterations...", end, iterations))
  }

  message("SOM training complete.")
  return(My_Model)
}

#' Load Data and Convert to a Matrix
#'
#' Loads data from a CSV file or an in-memory object (data frame or matrix),
#' optionally removes specified columns, and applies specified normalization methods
#' before converting the data to a matrix. In the original dataset, rows represent
#' observations (e.g., samples), columns represent variables (e.g., features),
#' and all cells (except for column headers and, if applicable, row headers)
#' must only contain numeric values.
#'
#' @importFrom utils read.csv
#' @param input Character or in-memory dataset. A string specifying the path to a CSV file, or an in-memory
#'   object (data frame or matrix). Argument is required.
#' @param remove_columns Integer, character, or vector of either.
#'   If specified, removes the columns of the dataset indicated by positions or names.
#'   This is useful, for example, when the first column contains
#'   non-numeric identifiers (e.g., sample names) that should be
#'   excluded from the analysis. Default is `NULL`.
#' @param remove_row_headings Deprecated, use `remove_columns = 1` instead. Logical. If `TRUE`, removes the first column of the dataset. Default is `FALSE`.
#' @param scaling Character. Normalization method to apply to the values in each column of the dataset. One of:
#'   \describe{
#'     \item{"no"}{No scaling is applied (default).}
#'     \item{"simpleFeature"}{Divided by the maximum value.}
#'     \item{"minMax"}{Scale values to range [0, 1].}
#'     \item{"zScore"}{standardize values by subtracting the mean and dividing by the standard deviation.}
#'   }
#' @return A matrix with the processed data.
#' @examples
#' # Example 1: Load toy data from a CSV file
#' file_path <- system.file("extdata", "toy_data.csv", package = "somhca")
#'
#' # Run the loadMatrix function with the mock data
#' myMatrix <- loadMatrix(file_path, remove_columns = 1, scaling = "minMax")
#'
#' # Example 2: Load from a toy data frame
#' df <- data.frame(
#'   ID = paste0("Sample", 1:100), # Character column for row headings
#'   matrix(rnorm(900), nrow = 100, ncol = 9) # Numeric data
#' )
#'
#' # Run the loadMatrix function with the mock data
#' myMatrix <- loadMatrix(df, remove_columns = 1, scaling = "zScore")
#'
#' # Example 3: Load from a toy matrix
#' mat <- matrix(rnorm(900), nrow = 100, ncol = 9) # Numeric data
#'
#' # Run the loadMatrix function with the mock data
#' myMatrix <- loadMatrix(mat, scaling = "simpleFeature")
#' @export

loadMatrix <- function(input, remove_columns = NULL, remove_row_headings = FALSE, scaling = "no") {

  # Determine input type and load data accordingly
  if (is.character(input)) {
    # If input is a file path, read the data
    data <- read.csv(input)
  } else if (is.matrix(input) || is.data.frame(input)) {
    # If input is a matrix or data frame, use it directly
    data <- as.data.frame(input)  # Convert matrix to data frame if necessary
  } else {
    stop("Input must be a file path (character), data frame, or matrix.")
  }

  # Validate scaling argument
  valid_scaling <- c("no", "simpleFeature", "minMax", "zScore")
  if (!scaling %in% valid_scaling) {
    stop(
      "`scaling` must be one of: ",
      paste(valid_scaling, collapse = ", ")
    )
  }

  # Handle deprecated argument
  if (!missing(remove_row_headings)) {
    .Deprecated(
      new = "remove_columns",
      msg = "`remove_row_headings` is deprecated. Use `remove_columns = 1` instead."
    )

    if (isTRUE(remove_row_headings)) {
      remove_columns <- unique(c(remove_columns, 1))
    }
  }

  # Remove specified columns
  if (!is.null(remove_columns)) {

    # If character, convert column names to indices
    if (is.character(remove_columns)) {
      missing_cols <- setdiff(remove_columns, colnames(data))
      if (length(missing_cols) > 0) {
        stop(
          "The following columns were not found in `data`: ",
          paste(missing_cols, collapse = ", ")
        )
      }
      remove_columns <- match(remove_columns, colnames(data))
    }

    # Validate numeric indices
    if (!is.numeric(remove_columns)) {
      stop("`remove_columns` must be numeric or character.")
    }

    if (any(remove_columns < 1 | remove_columns > ncol(data))) {
      stop("`remove_columns` contains invalid column indices.")
    }

    data <- data[, -remove_columns, drop = FALSE]
  }

  if (!all(vapply(data, is.numeric, logical(1)))) {
    stop("All columns must be numeric after column removal.")
  }

  # Perform data scaling if specified
  if (scaling == "simpleFeature") {
    data[] <- lapply(data, function(x) x / max(x, na.rm = TRUE))
  } else if (scaling == "minMax") {
    data[] <- lapply(data, function(x)
      (x - min(x, na.rm = TRUE)) /
        (max(x, na.rm = TRUE) - min(x, na.rm = TRUE))
    )
  } else if (scaling == "zScore") {
    data[] <- lapply(data, scale)
  }

  # Convert the data to a matrix
  data_matrix <- as.matrix(data)

  return(data_matrix)
}

#' Perform Clustering on SOM Nodes
#'
#' Groups similar nodes of a SOM using hierarchical clustering.
#' By default, the optimal number of clusters is determined automatically using the KGS penalty function,
#' but the user can also specify a fixed number of clusters.
#'
#' @import kohonen
#' @importFrom maptree kgs
#' @importFrom grDevices colorRampPalette
#' @importFrom RColorBrewer brewer.pal
#' @importFrom stats dist hclust cutree
#' @importFrom utils read.csv
#' @importFrom fpc cluster.stats
#' @param model In-memory SOM model object. A trained SOM model. Argument is required.
#' @param n_clusters Integer.
#'   If provided, specifies the number of clusters to cut the SOM dendrogram into.
#'   If NULL (default), the optimal number of clusters is determined automatically
#'   using the KGS penalty function.
#' @param validity_indices Logical. Indicates whether to compute and print popular clustering validity indices (Silhouette, Dunn, Calinski-Harabasz, Pearson Gamma). Default is `TRUE`.
#' @param plot_result Logical. Indicates whether to plot the clustering result. Default is `TRUE`.
#' @param input Character or in-memory dataset. A string specifying the path to a CSV file, or an in-memory
#'   object (data frame or matrix). Default is `NULL`.
#'   If provided, cluster assignments are appended to the observations in the original dataset, and the updated data is stored in a package environment as 'DataAndClusters'.
#' @return
#'   Invisibly returns `NULL`. If `plot_result = TRUE`, a plot of the clusters on the SOM grid is produced.
#'   If `input` is provided, the clustered dataset is stored in the package
#'   environment and can be retrieved with `getClusterData()`.
#' @examples
#' # Create a toy matrix with 9 columns and 100 rows
#' data <- matrix(rnorm(900), ncol = 9, nrow = 100)  # 900 random numbers, 100 rows, 9 columns
#'
#' # Run the finalSOM function with the mock data
#' model <- finalSOM(data, dimension = 6, iterations = 700)
#'
#' # Example 1: Perform clustering using the mock model
#' clusterSOM(model, plot_result = TRUE)
#'
#' # Example 2: Assign SOM-based clusters to an in-memory data frame
#' df <- data.frame(
#'   ID = paste0("Sample", 1:100), # Character column for row headings
#'   matrix(rnorm(900), ncol = 9, nrow = 100) # Numeric data
#' )
#' clusterSOM(model, plot_result = FALSE, input = df)
#' getClusterData()
#'
#' # Example 3: Load toy data from a CSV file, perform clustering, and retrieve the clustered dataset
#' file_path <- system.file("extdata", "toy_data.csv", package = "somhca")
#' clusterSOM(model, plot_result = FALSE, input = file_path)
#' getClusterData()
#' @export

clusterSOM <- function(model, n_clusters = NULL, validity_indices = TRUE, plot_result = TRUE, input = NULL) {
  # Validate model input
  if (!inherits(model, "kohonen")) {
    stop("The input model must be a trained SOM object (of class 'kohonen').")
  }

  # Validate and load the input data
  if (is.null(input)) {
    data <- NULL
  } else if (is.character(input)) {
    if (!file.exists(input)) {
      stop("The specified file path does not exist or cannot be read.")
    }
    data <- read.csv(input)
  } else if (is.matrix(input) || is.data.frame(input)) {
    data <- as.data.frame(input)  # Convert matrix to data frame if necessary
  } else {
    stop("Input must be a file path (character), data frame, or matrix.")
  }

  # Perform hierarchical clustering
  codes <- getCodes(model)
  distance <- dist(codes, method = "euclidean")
  clustering <- hclust(distance, method = "ward.D2")

  # Determine optimal number of clusters using the KGS penalty function
  if (is.null(n_clusters)) {
    optimal_k <- kgs(clustering, distance, maxclust = 20)
    clusters <- as.integer(names(optimal_k[which.min(optimal_k)]))
    message(clusters, " clusters were determined using KGS.\n")
  } else {
    if (!is.numeric(n_clusters) ||
        length(n_clusters) != 1 ||
        n_clusters < 1) {
      stop("`n_clusters` must be a single positive integer.")
    }
    clusters <- as.integer(n_clusters)
    message(clusters, " clusters were specified by the user.\n")
  }

  n_units <- nrow(codes)

  if (clusters > n_units) {
    stop(
      "`n_clusters` (", clusters,
      ") cannot exceed the number of SOM units (", n_units, ")."
    )
  }

  # Assign clusters to SOM units
  som_cluster <- cutree(clustering, clusters)

  # Cluster validity indices (informative message)
  if (isTRUE(validity_indices)) {
    if (requireNamespace("fpc", quietly = TRUE)) {
      stats <- fpc::cluster.stats(distance, som_cluster)

      message(
        "Cluster validity indices:\n",
        sprintf("  %-25s %6.3f (higher is better; >0.25 reasonable, >0.5 strong)\n",
                "Average Silhouette Width:", stats$avg.silwidth),
        sprintf("  %-25s %6.3f (higher is better)\n",
                "Dunn Index:", stats$dunn),
        sprintf("  %-25s %6.1f (higher is better)\n",
                "Calinski-Harabasz Index:", stats$ch),
        sprintf("  %-25s %6.3f (closer to 1 is better)",
                "Pearson Gamma Coefficient:", stats$pearsongamma)
      )
    } else {
      message("Package 'fpc' not installed; cluster validity indices skipped.")
    }
  }

  # Create a color palette
  max_colors <- max(20, clusters)  # Ensure enough colors
  pretty_palette <- colorRampPalette(brewer.pal(8, "Set1"))(max_colors)

  # Plot the result if requested
  if (plot_result) {
    plot(model, type = "mapping", bgcol = pretty_palette[som_cluster], main = "Clusters")
    add.cluster.boundaries(model, som_cluster)
  }

  # Process and store the data if input is provided
  if (!is.null(data)) {

    # Validate that input matches SOM training data
    if (nrow(data) != length(model$unit.classif)) {
      stop(
        "The number of rows in `input` (", nrow(data),
        ") does not match the number of observations used to train the SOM (",
        length(model$unit.classif), ")."
      )
    }

    # Map clusters to original observations
    cluster_assignment <- som_cluster[model$unit.classif]

    # Add cluster assignments to the dataset
    data$Cluster <- factor(cluster_assignment)
    data <- data[, c("Cluster", setdiff(names(data), "Cluster"))]

    # Store the data in the package environment
    somhca_env$DataAndClusters <- data

    # Notify the user
    message("The clustered dataset is stored in the package environment as 'DataAndClusters'. Use `getClusterData()` to retrieve it.\n")
  }

  # Return invisibly
  invisible(NULL)
}

#' Retrieve Clustered Data
#'
#' Access the dataset with cluster assignments stored by `clusterSOM` or `ClusterX`.
#' @return A data frame with the clustered dataset.
#' @export
getClusterData <- function() {
  if (!exists("DataAndClusters", envir = somhca_env)) {
    stop("No clustered data found. Run `clusterSOM` with a valid `input` first.")
  }
  return(somhca_env$DataAndClusters)
}

#' Estimate Optimal SOM Grid Size based on the Data
#'
#' Computes the optimal grid size for training a SOM using various quality
#' measures and heuristic approaches.
#'
#' @importFrom kohonen som somgrid
#' @importFrom aweSOM somQuality
#' @import dplyr
#' @importFrom stats sd
#' @importFrom utils txtProgressBar setTxtProgressBar
#'
#' @param data Matrix containing numeric data. A preprocessed data matrix containing the input data for SOM training. Argument is required.
#' @param method Character or integer. Method for estimating the maximum grid dimension. One of:
#'   \describe{
#'     \item{"A"}{Uses the heuristic formula by Vesanto et al. (default).}
#'     \item{"B"}{Applies an alternative heuristic approach.}
#'     \item{numeric}{Manually specified maximum dimension.}
#'   }
#' @param increments Integer. Step size for increasing grid dimensions.
#'   For example, set increments to 2 or 5 to increment the grid size by 2 or 5 rows/columns at each step.
#'   Smaller increments lead to more granular searches but may increase computation time;
#'   larger increments risk errors if they exceed the estimated maximum SOM grid dimensions.
#'   Default is 1.
#' @param iterations Integer. Number of iterations (`rlen`) for SOM training.
#'   If set to NULL (default), the function automatically calculates a sensible number
#'   of iterations based on the dataset size.
#'   If you want to override this, you can provide a numeric value.
#'   A lower value, such as less than 500, helps reduce computation time—if the process takes too long or an error occurs, try reducing the number of iterations for quicker results.
#'
#' @return A data frame summarizing optimal SOM grid dimensions and suggested iterations for each quality measure.
#'   Use these results to select the most suitable grid size for your SOM.
#'
#' @examples
#' # Create a toy matrix with 9 columns and 100 rows
#' data <- matrix(rnorm(900), ncol = 9, nrow = 100)  # 900 random numbers, 100 rows, 9 columns
#'
#' # Run the optimalSOM function with the mock data
#' myOptimalSOM <- optimalSOM(data, method = "A", increments = 2, iterations = 300)
#'
#' @export

optimalSOM <- function(data, method = "A", increments = 1, iterations = NULL) {

  # Basic safety checks
  data <- as.matrix(data)
  stopifnot(nrow(data) > 1, ncol(data) > 1)
  stopifnot(is.numeric(increments), increments >= 1)

  # Helper functions to calculate default rlen based on data rows
  estimate_rlen <- function(data,
                            passes = NULL,
                            min_rlen = 10,
                            max_rlen = 100) {

    n <- nrow(data)

    # Automatic default:
    # larger datasets need fewer passes
    if (is.null(passes)) {
      passes <- if (n < 1000) {
        50
      } else if (n < 10000) {
        30
      } else if (n < 100000) {
        20
      } else {
        10
      }
    }

    rlen <- round(passes)

    max(min_rlen, min(rlen, max_rlen))
  }

  qe_converged <- function(qe_hist,
                           epsilon = max(1e-6, 0.01 * sd(diff(qe_hist))),
                           warmup = 3) {

    k <- length(qe_hist)

    if (k <= warmup)
      return(FALSE)

    rel_improve <- (qe_hist[k-1] - qe_hist[k]) / qe_hist[k-1]

    rel_improve < epsilon
  }

  # Calculate the maximum dimension of a square grid size
  if(method=="A") {
    max_dim <- round(sqrt(5*sqrt(nrow(data)))) # Vesanto heuristic
  }
  else if(method=="B") {
    max_dim <- round((nrow(data))^(1/2.5)) # Alternative heuristic
  }
  else if(is.numeric(method)) {
    max_dim <- round(method) # Manual selection
  } else {
    stop("method must be 'A', 'B', or a numeric value")
  }

  if (max_dim < 2) {
    stop("Maximum grid dimension must be at least 2")
  }

  # Initialize dataframe to store SOM quality metrics
  error_df <- dplyr::tibble(
    Dimension = integer(),
    Quantisation_error = numeric(),
    Topographic_error = numeric(),
    `Kaski-Lagus_error` = numeric(),
    Explained_variance = numeric()
  )

  seq_dims <- seq(2, max_dim, increments)
  progbar <- txtProgressBar(min = 0, max = length(seq_dims), style = 3)

  qe_values <- numeric(length(seq_dims))
  qe_history <- numeric()
  no_improve <- 0
  printed_message <- FALSE

  for (idx in seq_along(seq_dims)) {

    i <- seq_dims[idx]

    My_Grid <- somgrid(
      xdim = i,
      ydim = i,
      topo = "hexagonal",
      toroidal = TRUE
    )

    # Determine rlen: use user-specified iterations or calculate automatically
    rlen_to_use <- if (is.null(iterations)) {
      rlen_auto <- estimate_rlen(data)
      if (!printed_message && interactive()) {
        message(sprintf(
          "Iterations automatically set to %d.",
          rlen_auto
        ))
        printed_message <- TRUE
      }
      rlen_auto
    } else {
      if (!printed_message && interactive()) {
        message(sprintf(
          "Using user-specified iterations: %d.",
          iterations
        ))
        printed_message <- TRUE
      }
      iterations
    }

    # Train SOM
    My_Model <- som(
      X = data,
      grid = My_Grid,
      rlen = rlen_to_use,
      alpha = c(0.05, 0.01),
      keep.data = TRUE
    )

    # Compute quality metrics
    sq <- aweSOM::somQuality(My_Model, data)

    qe_values[idx] <- as.numeric(sq$err.quant)

    error_df <- error_df %>%
      dplyr::add_row(
        Dimension = i,
        Quantisation_error = qe_values[idx],
        Topographic_error  = as.numeric(sq$err.topo),
        `Kaski-Lagus_error` = as.numeric(sq$err.kaski),
        Explained_variance = as.numeric(sq$err.varratio)
      )

    # QE plateau stopping rule
    qe_history <- c(qe_history, qe_values[idx])

    if (idx > 3) {

      if (qe_converged(qe_history)) {
        no_improve <- no_improve + 1
      } else {
        no_improve <- 0
      }

      if (no_improve >= 3) {
        message(
          sprintf(
            "QE plateau reached at grid %dx%d (delta QE < 0.2%%). Stopping grid expansion.",
            i, i
          )
        )
        utils::setTxtProgressBar(progbar, idx)
        break
      }
    }

    utils::setTxtProgressBar(progbar, idx)
  }

  close(progbar)

  # Standardize and compute combined error metrics
  error_df <- error_df %>%
    dplyr::mutate(
      dplyr::across(
        c(
          Quantisation_error,
          Topographic_error,
          `Kaski-Lagus_error`,
          Explained_variance
        ),
        ~ as.numeric(scale(.x))
      ),
      QplusT_error = Quantisation_error + Topographic_error,
      QplusTplusK_error = Quantisation_error + Topographic_error + `Kaski-Lagus_error`,
      all_error = Explained_variance - QplusTplusK_error
    )

  # Determine optimal grid for each quality measure
  QTerror_df  <- error_df %>% dplyr::slice_min(QplusT_error, n = 1, with_ties = TRUE)
  Kerror_df   <- error_df %>% dplyr::slice_min(`Kaski-Lagus_error`, n = 1, with_ties = TRUE)
  QTKerror_df <- error_df %>% dplyr::slice_min(QplusTplusK_error, n = 1, with_ties = TRUE)
  Verror_df   <- error_df %>% dplyr::slice_max(Explained_variance, n = 1, with_ties = TRUE)
  All_errors_df <- error_df %>% dplyr::slice_max(all_error, n = 1, with_ties = TRUE)

  # Extract dimensions
  My_dim_QT  <- QTerror_df$Dimension
  My_dim_K   <- Kerror_df$Dimension
  My_dim_QTK <- QTKerror_df$Dimension
  My_dim_V   <- Verror_df$Dimension
  My_dim_all <- All_errors_df$Dimension

  # Calculate rlen for each optimal dimension
  estimate_rlen_final <- function(data, grid_dim, min_rlen = 500, max_rlen = 5000) {
    n <- nrow(data)
    n_neurons <- grid_dim^2  # assuming square grid
    rlen <- round(n * log10(n_neurons))
    max(min_rlen, min(rlen, max_rlen))
  }

  rlen_QT  <- estimate_rlen_final(data, My_dim_QT)
  rlen_K   <- estimate_rlen_final(data, My_dim_K)
  rlen_QTK <- estimate_rlen_final(data, My_dim_QTK)
  rlen_V   <- estimate_rlen_final(data, My_dim_V)
  rlen_all <- estimate_rlen_final(data, My_dim_all)

  # Return dataframe with optimal grid dimensions and suggested rlen
  return(data.frame(
    "Quality measure" = c(
      "Min nQe+nTe",
      "Min nKLe",
      "Min nQe+nTe+nKLe",
      "Max n%ev",
      "Max QI"
    ),
    "Value" = c(
      QTerror_df$QplusT_error,
      Kerror_df$`Kaski-Lagus_error`,
      QTKerror_df$QplusTplusK_error,
      Verror_df$Explained_variance,
      All_errors_df$all_error
    ),
    "Associated grid dimension" = c(
      My_dim_QT,
      My_dim_K,
      My_dim_QTK,
      My_dim_V,
      My_dim_all
    ),
    "Suggested iterations" = c(
      rlen_QT,
      rlen_K,
      rlen_QTK,
      rlen_V,
      rlen_all
    ),
    check.names = FALSE
  ))
}

#' Generate SOM Visualization Plots
#'
#' Creates various types of plots to visualize and evaluate the trained SOM model.
#'
#' @import kohonen
#' @param model In-memory SOM model object. A trained SOM model. Argument is required.
#' @param plot_type Integer. Specifies the type of plot to generate. One of:
#'   \describe{
#'     \item{1}{Training progress plot (changes during training).}
#'     \item{2}{Node count plot (number of samples mapped to each node) for assessing map quality.}
#'     \item{3}{U-matrix plot (similarities between neighboring nodes).}
#'     \item{4}{Weight vector plot (patterns in the distributions of variables).}
#'     \item{5}{Kohonen heatmaps for all variables in the dataset (distribution of single variables across the map).}
#'   }
#'   Argument is required.
#' @param data Matrix containing numeric data. A preprocessed data matrix containing the input data for SOM training. Default is `NULL` (required only for `plot_type = 5`).
#' @return A plot or a series of plots is generated and displayed based on the specified type.
#' @examples
#' # Create a toy matrix with 9 columns and 100 rows
#' data <- matrix(rnorm(900), ncol = 9, nrow = 100)  # 900 random numbers, 100 rows, 9 columns
#'
#' # Assign column names to the data matrix
#' colnames(data) <- paste("Var", 1:ncol(data), sep = "_")
#'
#' # Run the finalSOM function with the mock data
#' model <- finalSOM(data, dimension = 6, iterations = 700)
#'
#' # Generate plots using the mock model
#' generatePlot(model, plot_type = 2)
#' generatePlot(model, plot_type = 5, data)
#' @export

generatePlot <- function(model, plot_type, data = NULL) {

  if (!inherits(model, "kohonen")) {
    stop("`model` must be a trained SOM object from the kohonen package.")
  }

  # Select plot based on plot_type
  if (plot_type == 1) {
    plot(model, type = "changes")

  } else if (plot_type == 2) {
    plot(model, type = "count", main = "Node Counts")

  } else if (plot_type == 3) {
    plot(model, type = "dist.neighbours", main = "SOM Neighbour Distances")

  } else if (plot_type == 4) {
    plot(model, type = "codes", main = "Weight Vectors")

  } else if (plot_type == 5) {

    if (is.null(data)) {
      stop("For plot_type = 5, `data` must be provided.")
    }

    codes <- getCodes(model)

    if (ncol(data) != ncol(codes)) {
      stop("Number of columns in `data` must match the SOM model variables.")
    }

    var_names <- colnames(codes)
    if (is.null(var_names)) {
      var_names <- paste0("Var_", seq_len(ncol(codes)))
    }

    for (i in seq_len(ncol(codes))) {
      plot(model, type = "property", property = codes[, i], main = var_names[i])
    }

  } else {
    stop("Invalid `plot_type`. Must be an integer between 1 and 5.")
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loadMatrix.R
\name{loadMatrix}
\alias{loadMatrix}
\title{Load Data and Convert to a Matrix}
\usage{
loadMatrix(
  input,
  remove_columns = NULL,
  remove_row_headings = FALSE,
  scaling = "no"
)
}
\arguments{
\item{input}{Character or in-memory dataset. A string specifying the path to a CSV file, or an in-memory
object (data frame or matrix). Argument is required.}

\item{remove_columns}{Integer, character, or vector of either.
If specified, removes the columns of the dataset indicated by positions or names.
This is useful, for example, when the first column contains
non-numeric identifiers (e.g., sample names) that should be
excluded from the analysis. Default is `NULL`.}

\item{remove_row_headings}{Deprecated, use `remove_columns = 1` instead. Logical. If `TRUE`, removes the first column of the dataset. Default is `FALSE`.}

\item{scaling}{Character. Normalization method to apply to the values in each column of the dataset. One of:
\describe{
  \item{"no"}{No scaling is applied (default).}
  \item{"simpleFeature"}{Divided by the maximum value.}
  \item{"minMax"}{Scale values to range [0, 1].}
  \item{"zScore"}{standardize values by subtracting the mean and dividing by the standard deviation.}
}}
}
\value{
A matrix with the processed data.
}
\description{
Loads data from a CSV file or an in-memory object (data frame or matrix),
optionally removes specified columns, and applies specified normalization methods
before converting the data to a matrix. In the original dataset, rows represent
observations (e.g., samples), columns represent variables (e.g., features),
and all cells (except for column headers and, if applicable, row headers)
must only contain numeric values.
}
\examples{
# Example 1: Load toy data from a CSV file
file_path <- system.file("extdata", "toy_data.csv", package = "somhca")

# Run the loadMatrix function with the mock data
myMatrix <- loadMatrix(file_path, remove_columns = 1, scaling = "minMax")

# Example 2: Load from a toy data frame
df <- data.frame(
  ID = paste0("Sample", 1:100), # Character column for row headings
  matrix(rnorm(900), nrow = 100, ncol = 9) # Numeric data
)

# Run the loadMatrix function with the mock data
myMatrix <- loadMatrix(df, remove_columns = 1, scaling = "zScore")

# Example 3: Load from a toy matrix
mat <- matrix(rnorm(900), nrow = 100, ncol = 9) # Numeric data

# Run the loadMatrix function with the mock data
myMatrix <- loadMatrix(mat, scaling = "simpleFeature")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimalSOM.R
\name{optimalSOM}
\alias{optimalSOM}
\title{Estimate Optimal SOM Grid Size based on the Data}
\usage{
optimalSOM(data, method = "A", increments = 1, iterations = NULL)
}
\arguments{
\item{data}{Matrix containing numeric data. A preprocessed data matrix containing the input data for SOM training. Argument is required.}

\item{method}{Character or integer. Method for estimating the maximum grid dimension. One of:
\describe{
  \item{"A"}{Uses the heuristic formula by Vesanto et al. (default).}
  \item{"B"}{Applies an alternative heuristic approach.}
  \item{numeric}{Manually specified maximum dimension.}
}}

\item{increments}{Integer. Step size for increasing grid dimensions.
For example, set increments to 2 or 5 to increment the grid size by 2 or 5 rows/columns at each step.
Smaller increments lead to more granular searches but may increase computation time;
larger increments risk errors if they exceed the estimated maximum SOM grid dimensions.
Default is 1.}

\item{iterations}{Integer. Number of iterations (`rlen`) for SOM training.
If set to NULL (default), the function automatically calculates a sensible number
of iterations based on the dataset size.
If you want to override this, you can provide a numeric value.
A lower value, such as less than 500, helps reduce computation time—if the process takes too long or an error occurs, try reducing the number of iterations for quicker results.}
}
\value{
A data frame summarizing optimal SOM grid dimensions and suggested iterations for each quality measure.
  Use these results to select the most suitable grid size for your SOM.
}
\description{
Computes the optimal grid size for training a SOM using various quality
measures and heuristic approaches.
}
\examples{
# Create a toy matrix with 9 columns and 100 rows
data <- matrix(rnorm(900), ncol = 9, nrow = 100)  # 900 random numbers, 100 rows, 9 columns

# Run the optimalSOM function with the mock data
myOptimalSOM <- optimalSOM(data, method = "A", increments = 2, iterations = 300)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterX.R
\name{clusterX}
\alias{clusterX}
\title{Perform Hierarchical Clustering on Data Objects}
\usage{
clusterX(
  x,
  n_clusters = NULL,
  validity_indices = TRUE,
  plot_result = TRUE,
  input = NULL
)
}
\arguments{
\item{x}{Data frame or matrix containing numeric data to be clustered (non-numeric columns are automatically ignored). Argument is required.}

\item{n_clusters}{Integer.
If provided, specifies the number of clusters to cut the dendrogram into.
If NULL (default), the optimal number of clusters is determined automatically
using the KGS penalty function.}

\item{validity_indices}{Logical. Indicates whether to compute and print popular clustering validity indices (Silhouette, Dunn, Calinski-Harabasz, Pearson Gamma). Default is `TRUE`.}

\item{plot_result}{Logical. Indicates whether to plot the clustering result. Default is `TRUE`.}

\item{input}{Character or in-memory dataset. A string specifying the path to a CSV file, or an in-memory
object (data frame or matrix). Default is `NULL`.
If provided, cluster assignments are appended to the observations in the original dataset,
and the updated data is stored in a package environment as 'DataAndClusters'.}
}
\value{
Invisibly returns `NULL`. If `plot_result = TRUE`, a plot of the clustering result is produced.
  If `input` is provided, the clustered dataset is stored in the package
  environment and can be retrieved with `getClusterData()`.
}
\description{
Groups similar observations using hierarchical clustering
This function is designed for clustering data types other than trained SOM models.
By default, the optimal number of clusters is determined automatically using the KGS penalty function,
but the user can also specify a fixed number of clusters.
}
\examples{
# Create a toy matrix with 9 columns and 100 rows
data <- matrix(rnorm(900), ncol = 9, nrow = 100)

# Example 1: Perform clustering directly on the matrix
clusterX(data, plot_result = TRUE)

# Example 2: Assign clusters to an in-memory data frame
df <- data.frame(
  ID = paste0("Sample", 1:100), # Character column for row headings
  matrix(rnorm(900), ncol = 9, nrow = 100) # Numeric data
)
clusterX(data, plot_result = FALSE, input = df)
getClusterData()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/finalSOM.R
\name{finalSOM}
\alias{finalSOM}
\title{Re-Train SOM Model}
\usage{
finalSOM(data, dimension, iterations, chunk = 100)
}
\arguments{
\item{data}{Matrix containing numeric data. A preprocessed data matrix containing the input data for SOM training. Argument is required.}

\item{dimension}{Integer. Dimension of the square SOM grid (e.g., 5 results in a 5x5 grid). Argument is required.}

\item{iterations}{Integer. Number of iterations for training the SOM model. Use a large value, e.g., 500 or higher, for improved training (an error message could suggest that reducing the number of iterations might be necessary).
For larger grids, more iterations may be required to ensure convergence. Reducing iterations may speed training but risk under-trained neurons.  Argument is required.}

\item{chunk}{Integer. Number of iterations per training block.
The SOM will be trained in chunks of this many iterations, with a progress
message printed after each block. This helps notify the user that the function
is running and not frozen. Larger values reduce the frequency
of messages; smaller values provide more frequent updates but may slightly
slow execution. Default is 100.}
}
\value{
A trained SOM model object.
}
\description{
Re-trains the SOM using a specified optimal grid size and number of iterations.
}
\examples{
# Create a toy matrix with 9 columns and 100 rows
data <- matrix(rnorm(900), ncol = 9, nrow = 100)  # 900 random numbers, 100 rows, 9 columns

# Run the finalSOM function with the mock data
myFinalSOM <- finalSOM(data, dimension = 6, iterations = 700)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterSOM.R
\name{clusterSOM}
\alias{clusterSOM}
\title{Perform Clustering on SOM Nodes}
\usage{
clusterSOM(
  model,
  n_clusters = NULL,
  validity_indices = TRUE,
  plot_result = TRUE,
  input = NULL
)
}
\arguments{
\item{model}{In-memory SOM model object. A trained SOM model. Argument is required.}

\item{n_clusters}{Integer.
If provided, specifies the number of clusters to cut the SOM dendrogram into.
If NULL (default), the optimal number of clusters is determined automatically
using the KGS penalty function.}

\item{validity_indices}{Logical. Indicates whether to compute and print popular clustering validity indices (Silhouette, Dunn, Calinski-Harabasz, Pearson Gamma). Default is `TRUE`.}

\item{plot_result}{Logical. Indicates whether to plot the clustering result. Default is `TRUE`.}

\item{input}{Character or in-memory dataset. A string specifying the path to a CSV file, or an in-memory
object (data frame or matrix). Default is `NULL`.
If provided, cluster assignments are appended to the observations in the original dataset, and the updated data is stored in a package environment as 'DataAndClusters'.}
}
\value{
Invisibly returns `NULL`. If `plot_result = TRUE`, a plot of the clusters on the SOM grid is produced.
  If `input` is provided, the clustered dataset is stored in the package
  environment and can be retrieved with `getClusterData()`.
}
\description{
Groups similar nodes of a SOM using hierarchical clustering.
By default, the optimal number of clusters is determined automatically using the KGS penalty function,
but the user can also specify a fixed number of clusters.
}
\examples{
# Create a toy matrix with 9 columns and 100 rows
data <- matrix(rnorm(900), ncol = 9, nrow = 100)  # 900 random numbers, 100 rows, 9 columns

# Run the finalSOM function with the mock data
model <- finalSOM(data, dimension = 6, iterations = 700)

# Example 1: Perform clustering using the mock model
clusterSOM(model, plot_result = TRUE)

# Example 2: Assign SOM-based clusters to an in-memory data frame
df <- data.frame(
  ID = paste0("Sample", 1:100), # Character column for row headings
  matrix(rnorm(900), ncol = 9, nrow = 100) # Numeric data
)
clusterSOM(model, plot_result = FALSE, input = df)
getClusterData()

# Example 3: Load toy data from a CSV file, perform clustering, and retrieve the clustered dataset
file_path <- system.file("extdata", "toy_data.csv", package = "somhca")
clusterSOM(model, plot_result = FALSE, input = file_path)
getClusterData()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterSOM.R
\name{getClusterData}
\alias{getClusterData}
\title{Retrieve Clustered Data}
\usage{
getClusterData()
}
\value{
A data frame with the clustered dataset.
}
\description{
Access the dataset with cluster assignments stored by `clusterSOM` or `ClusterX`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generatePlot.R
\name{generatePlot}
\alias{generatePlot}
\title{Generate SOM Visualization Plots}
\usage{
generatePlot(model, plot_type, data = NULL)
}
\arguments{
\item{model}{In-memory SOM model object. A trained SOM model. Argument is required.}

\item{plot_type}{Integer. Specifies the type of plot to generate. One of:
\describe{
  \item{1}{Training progress plot (changes during training).}
  \item{2}{Node count plot (number of samples mapped to each node) for assessing map quality.}
  \item{3}{U-matrix plot (similarities between neighboring nodes).}
  \item{4}{Weight vector plot (patterns in the distributions of variables).}
  \item{5}{Kohonen heatmaps for all variables in the dataset (distribution of single variables across the map).}
}
Argument is required.}

\item{data}{Matrix containing numeric data. A preprocessed data matrix containing the input data for SOM training. Default is `NULL` (required only for `plot_type = 5`).}
}
\value{
A plot or a series of plots is generated and displayed based on the specified type.
}
\description{
Creates various types of plots to visualize and evaluate the trained SOM model.
}
\examples{
# Create a toy matrix with 9 columns and 100 rows
data <- matrix(rnorm(900), ncol = 9, nrow = 100)  # 900 random numbers, 100 rows, 9 columns

# Assign column names to the data matrix
colnames(data) <- paste("Var", 1:ncol(data), sep = "_")

# Run the finalSOM function with the mock data
model <- finalSOM(data, dimension = 6, iterations = 700)

# Generate plots using the mock model
generatePlot(model, plot_type = 2)
generatePlot(model, plot_type = 5, data)
}
